#version 330

out vec4 fragColor;

uniform vec2 res;
uniform float time;

const float sharpness = 300;

const float PI = 3.14159265358979323846;


mat2 rot(float a) {
        return mat2(
                cos(a), -sin(a),
                sin(a), cos(a)
        );
}

float circle(vec2 m, float r, vec2 p) {
        vec2 diff = m - p;
        float d = sqrt(dot(diff, diff));

        return clamp((r - d)*sharpness, 0.0, 1.0);
}

float circle_d(vec2 m, float r, vec2 p) {
        return circle(m, r, p) - circle(m, r - 1/sharpness, p);
}

float square(vec2 m, float a, float r, vec2 p) {
        vec2 diff = abs((m - p) * rot(r));
        float d = max(diff.x, diff.y);

        return clamp((a - d)*sharpness, 0.0, 1.0);
}

void main(void) {
        vec2 m = res/2;
        float s = min(m.x, m.y);
        vec2 p = (gl_FragCoord.xy - m) / s;

        float scene = 0;

        int n = 20;
        float b = 1;
        float alpha = 0;
        for (int i = 0; i < n; i++) {
                float c;

                c = circle_d(vec2(0, 0), b, p);
                scene = abs(scene - c);

                float delta = mod(time/5000, PI/2);
                alpha = mod(alpha + delta, PI/2);
                b /= sqrt(2);

                c = square(vec2(0, 0), b, alpha, p);
                scene = abs(scene - c);

                c = square(vec2(0, 0), b, -alpha, p);
                scene = abs(scene - c);
        }

        fragColor = vec4(vec3(1, 1, 1) * scene, 1);
}
